@@ -8,7 +8,7 @@ use Dumpvalue;
 use version ();
 use File::Spec ();
 
-our $VERSION = '0.28';
+our $VERSION = '0.30';
 our $VERBOSE = 0;
 our $ALLOW_DEV_VERSION = 0;
 our $FORK = 0;
@@ -197,7 +197,7 @@ sub _parse_version {
         } else {
             # XXX Limit Resources too
 
-            my($comp) = Safe->new("_pause::mldistwatch");
+            my($comp) = Safe->new;
             my $eval = qq{
                 local(\$^W) = 0;
                 Parse::PMFile::_parse_version_safely("$pmcp");
@@ -226,9 +226,10 @@ sub _parse_version {
                 if (ref $err) {
                     if ($err->{line} =~ /([\$*])([\w\:\']*)\bVERSION\b.*?\=(.*)/) {
                         local($^W) = 0;
-                        $self->_restore_overloaded_stuff if version->isa('version::vpp');
-                        $v = ($self->{UNSAFE} || $UNSAFE) ? eval $3 : $comp->reval($3);
-                        $v = $$v if $1 eq '*' && ref $v;
+                        my ($sigil, $vstr) = ($1, $3);
+                        $self->_restore_overloaded_stuff(1) if $err->{line} =~ /use\s+version\b/;
+                        $v = ($self->{UNSAFE} || $UNSAFE) ? eval $vstr : $comp->reval($vstr);
+                        $v = $$v if $sigil eq '*' && ref $v;
                     }
                     if ($@ or !$v) {
                         $self->_verbose(1, sprintf("reval failed: err[%s] for eval[%s]",
@@ -265,31 +266,46 @@ sub _parse_version {
 }
 
 sub _restore_overloaded_stuff {
-    my $self = shift;
+    my ($self, $used_version_in_safe) = @_;
     return if $self->{UNSAFE} || $UNSAFE;
 
     no strict 'refs';
     no warnings 'redefine';
 
     # version XS in CPAN
-    if (version->isa('version::vxs')) {
+    my $restored;
+    if ($INC{'version/vxs.pm'}) {
         *{'version::(""'} = \&version::vxs::stringify;
         *{'version::(0+'} = \&version::vxs::numify;
         *{'version::(cmp'} = \&version::vxs::VCMP;
         *{'version::(<=>'} = \&version::vxs::VCMP;
         *{'version::(bool'} = \&version::vxs::boolean;
+        $restored = 1;
+    }
     # version PP in CPAN
-    } elsif (version->isa('version::vpp')) {
+    if ($INC{'version/vpp.pm'}) {
         {
             package # hide from PAUSE
                 charstar;
             overload->import;
         }
-        *{'version::(""'} = \&version::vpp::stringify;
-        *{'version::(0+'} = \&version::vpp::numify;
-        *{'version::(cmp'} = \&version::vpp::vcmp;
-        *{'version::(<=>'} = \&version::vpp::vcmp;
-        *{'version::(bool'} = \&version::vpp::vbool;
+        if (!$used_version_in_safe) {
+            package # hide from PAUSE
+                version::vpp;
+            overload->import;
+        }
+        unless ($restored) {
+            *{'version::(""'} = \&version::vpp::stringify;
+            *{'version::(0+'} = \&version::vpp::numify;
+            *{'version::(cmp'} = \&version::vpp::vcmp;
+            *{'version::(<=>'} = \&version::vpp::vcmp;
+            *{'version::(bool'} = \&version::vpp::vbool;
+        }
+        *{'version::vpp::(""'} = \&version::vpp::stringify;
+        *{'version::vpp::(0+'} = \&version::vpp::numify;
+        *{'version::vpp::(cmp'} = \&version::vpp::vcmp;
+        *{'version::vpp::(<=>'} = \&version::vpp::vcmp;
+        *{'version::vpp::(bool'} = \&version::vpp::vbool;
         *{'charstar::(""'} = \&charstar::thischar;
         *{'charstar::(0+'} = \&charstar::thischar;
         *{'charstar::(++'} = \&charstar::increment;
@@ -301,8 +317,10 @@ sub _restore_overloaded_stuff {
         *{'charstar::(<=>'} = \&charstar::spaceship;
         *{'charstar::(bool'} = \&charstar::thischar;
         *{'charstar::(='} = \&charstar::clone;
+        $restored = 1;
+    }
     # version in core
-    } else {
+    if (!$restored) {
         *{'version::(""'} = \&version::stringify;
         *{'version::(0+'} = \&version::numify;
         *{'version::(cmp'} = \&version::vcmp;
